//-----------------------------------------------------------
//    fem_app.h
//-----------------------------------------------------------
#ifndef FEM_APP
#define FEM_APP
#include <algorithm>
#include <vector>
#include <tuple>
#include "sykes/multi_carray.h"
#include "sykes/fem_space.h"
#include "sykes/element/tetrahedron.h"
#include "sykes/material/mixed_material.h"
#include "sykes/boundary_condition.h"
#include "sykes/numeric/sym_banded_matrix.h"
#include "sykes/numeric/vector.h"

namespace fem_elastic3d{
    using sykes::realnum;
    
    struct fem_app
    {
        typedef sykes::fem::element::eltype_tet<1> element_type;
        typedef sykes::material::mixed_material<
            sykes::material::linear_elasticity<3>,
            sykes::material::density> material_type;
        typedef sykes::numeric::sym_banded_matrix<double> matrix_type;
        typedef sykes::numeric::vector<double> vector_type;
        
        sykes::fem::fem_space<
                element_type,
                material_type,
                matrix_type,
                vector_type
            > fem_space;
        
        inline void resize_total_matrix_and_vector()
        {
            std::size_t size1 = fem_space.get_node_array().size() * 3, half_band_width1 = 1;
            auto elements = fem_space.get_element_array();
            for(std::size_t i1 = 0, sz = elements.size(); i1 < sz; ++i1){
                auto const node_nums = elements[i1].get_node_nums();
                std::size_t diff = *std::max_element(node_nums.begin(), node_nums.end())
                    - *std::min_element(node_nums.begin(), node_nums.end());
                half_band_width1 = std::max(diff, half_band_width1);
            }
            half_band_width1 = (half_band_width1 + 1) * 3;
            fem_space.get_total_matrix().resize(size1, half_band_width1);
            fem_space.get_total_vector().resize(size1);
            fem_space.get_result_vector().resize(size1);
        }
    };
    
}
#endif

