//---- fem_elastic2d main.cpp

#include <vector>
#include <typeinfo>
#include <iostream>
#include <boost/numeric/ublas/io.hpp>

#include "sykes/solver/solver_CG.h"

#include "fem_app.h"
#include "sykes/equation/steady_linear_elastic_solid.h"
#include "sykes/boundary_condition.h"
#include "sykes/numeric/matrix_and_vector_output.h"
#include "input_output.h"


int main()
{
    namespace elastic_equation =
        sykes::fem::equation::steady_linear_elastic_solid;
    
    fem_elastic3d::fem_app fapp;
    sykes::multi_carray<sykes::realnum, 3> gravity{{0.0, 0.0, 0.0}};
    std::string filename, ofilename("test_out.vtk");
    
    std::cout << "filename:";
    std::cin >> filename;
    
    fem_elastic3d::read_input(fapp, filename);
    
    fapp.resize_total_matrix_and_vector();
    
    elastic_equation::compose_total_matrix_and_vector(
        gravity,
        fapp.fem_space);
    
    sykes::fem::boundary::treat_dirichlet_boundary_condition<3>(
        fapp.fem_space.get_total_matrix(),
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_dirichlet_boundary_condition());
    
    sykes::fem::boundary::treat_neumann_boundary_condition<3>(
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_neumann_boundary_condition());
    
    sykes::solver::conjugate_gradient_method(
        fapp.fem_space.get_total_matrix(),
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_result_vector(),
        1.0e-12);
    
    
    fem_elastic3d::output_result(fapp, ofilename);
}





