
#include <vector>
#include <typeinfo>
#include <iostream>
#include <boost/numeric/ublas/io.hpp>

#include "sykes/solver/solver_CG.h"

#include "fem_app.h"
#include "sykes/equation/steady_uncompressible_stokes_flow_mixed_interpolation.h"
#include "input_output.h"


int main()
{
    namespace stokes_equation =
        sykes::fem::equation::steady_uncompressible_stokes_flow_mixed_interpolation;
    
    test_stokes2d::fem_app fapp;
    sykes::multi_carray<sykes::realnum, 2> gravity{{0.0, 0.0}};
    
    std::string filename, ofilename("test_out.vtk");
    std::cout << "filename:";
    std::cin >> filename;
    test_stokes2d::read_input(fapp, filename);
    
    sykes::fem::reduce_band_width(fapp.fem_space);
    
    fapp.resize_total_matrix_and_vector();
    
    stokes_equation::compose_total_matrix_and_vector(
        gravity,
        fapp.fem_space);
    
    sykes::fem::boundary::treat_dirichlet_boundary_condition<3>(
        fapp.fem_space.get_total_matrix(),
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_dirichlet_boundary_condition());
    
    sykes::fem::boundary::treat_neumann_boundary_condition<3>(
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_neumann_boundary_condition());
    
    sykes::solver::conjugate_gradient_method(
        fapp.fem_space.get_total_matrix(),
        fapp.fem_space.get_total_vector(),
        fapp.fem_space.get_result_vector(),
        1.0e-8);
    
    test_stokes2d::output_result(fapp, ofilename);
}





