//-----------------------------------------------------------
//    boundary_condition.h
//-----------------------------------------------------------
#ifndef SYKES_FEM_BOUNDARY_CONDITION
#define SYKES_FEM_BOUNDARY_CONDITION

#include "sykes/value_type.h"

namespace sykes{ namespace fem{ namespace boundary{
    
    //-----------------------------------------------------------
    //    struct boundary_condition
    //-----------------------------------------------------------
    struct boundary_condition
    {
        std::size_t node_num;
        std::size_t axis;
        realnum value;
    };
    
    //-----------------------------------------------------------
    //    treat_dirichlet_boundary_condition
    //-----------------------------------------------------------
    template<std::size_t Dim, typename BoundaryArray, typename Mat, typename Vec>
    void treat_dirichlet_boundary_condition(
        Mat& _mat, Vec& _vec,
        BoundaryArray const& _bcond)
    {
        const std::size_t bsize = _bcond.size();
    
        for(std::size_t i0 = 0; i0 < bsize; ++i0){
            std::size_t bnum = _bcond[i0].node_num * Dim + _bcond[i0].axis;
        
            {
                auto row_a = _mat.begin1(), row_end = _mat.end1();
                while(row_a != row_end){
                    std::size_t rownum = row_a.index1();
                    if(rownum == bnum){
                        auto col_a = row_a.begin(), col_end = row_a.end();
                        --col_end;
                        while(col_a != col_end){
                            std::size_t colnum = col_a.index2();
                            _vec(colnum) -= (*col_a) * _bcond[i0].value;
                            *col_a = 0.0;
                            ++col_a;
                        }
                        auto rowb = col_a.begin(), end2 = col_a.end();
                        ++rowb;
                        while(rowb != end2){
                            std::size_t rownum2 = rowb.index1();
                            _vec(rownum2) -= (*rowb) * _bcond[i0].value;
                            *rowb = 0.0;
                            ++rowb;
                        }
                    }
                    ++row_a;
                }
            }
            _mat(bnum, bnum) = 1.0;
            _vec(bnum) = _bcond[i0].value;
        }
    }
    
    //-----------------------------------------------------------
    //    treat_neumann_boundary_condition
    //-----------------------------------------------------------
    template<std::size_t Dim, typename BoundaryArray, typename Vec>
    void treat_neumann_boundary_condition(
        Vec& _vec,
        BoundaryArray const& _bcond)
    {
        const std::size_t bsize = _bcond.size();
    
        for(std::size_t i0 = 0; i0 < bsize; ++i0){
            std::size_t bnum = _bcond[i0].node_num * Dim + _bcond[i0].axis;
            _vec(bnum) -= _bcond[i0].value;
        }
    }
}}}//---- namespace boundary fem sykes
#endif

