//-----------------------------------------------------------
//    eltype_triangle.h
//-----------------------------------------------------------
#ifndef SYKES_ELEMENT_DETAIL_ELTYPE_TRI
#define SYKES_ELEMENT_DETAIL_ELTYPE_TRI

#include <vector>
#include <cstddef>
#include <stdexcept>
#include "sykes/multi_carray.h"
#include "sykes/utility.h"

namespace sykes{ namespace fem{ namespace element{
    
    //-----------------------------------------------------------
    //    eltype_tri
    //-----------------------------------------------------------
    template<std::size_t Order>
    class eltype_tri
    {
    public:
        static std::size_t const order = Order;
        static std::size_t const vertex_size = 3;
        static std::size_t const node_size = combination<order + 2, 2>::value;
        static std::size_t const dim = 2;
        
        template<std::size_t N>
        class other_order
        {
        public:
            typedef eltype_tri<N> type;
        };
        
        eltype_tri() : m_node_num(), m_material_num() {}
        eltype_tri(std::vector<std::size_t> const& _nodes, std::size_t _mate = 0)
            : m_material_num(_mate)
        {
            if(_nodes.size() != node_size)
                throw std::invalid_argument("sykes::fem::element::eltype_tri::eltype_tri()");
            for(std::size_t i0 = 0; i0 < node_size; i0++)
                m_node_num[i0] = _nodes[i0];
        }
        
        multi_carray<std::size_t, node_size>& get_node_nums()
        { return m_node_num; }
        
        multi_carray<std::size_t, node_size> const& get_node_nums() const
        { return m_node_num; }
        
        std::size_t get_node_num(std::size_t _n) const
        { return m_node_num[_n]; }
        
        void set_node_nums(multi_carray<std::size_t, node_size> const& _nodes)
        { m_node_num = _nodes; }
        
        int get_material_num() const
        { return m_material_num; }
        
        void set_material_num(int _matenum)
        { m_material_num = _matenum; }
        
    private:
        multi_carray<std::size_t, node_size> m_node_num;
        int m_material_num;
    };
    
}}}//---- namespace element fem sykes
#endif

