//-----------------------------------------------------------
//    tri_interpolation_functions.h
//-----------------------------------------------------------
#ifndef SYKES_ELEMENT_TRI_INTERPOLATION_FUNCTIONS
#define SYKES_ELEMENT_TRI_INTERPOLATION_FUNCTIONS
#include "sykes/element/detail/interpolation_functions.h"
#include <tuple>

namespace sykes{ namespace fem{ namespace element{

    template<>
    realnum calc_region_integral<eltype_tri<1>>(multi_carray<realnum, 3, 2> const& _nodepos);

    template<>
    realnum calc_region_integral<eltype_tri<2>>(multi_carray<realnum, 6, 2> const& _nodepos);

    template<>
    realnum calc_region_integral<eltype_tri<3>>(multi_carray<realnum, 10, 2> const& _nodepos);

    template<>
    realnum calc_region_integral<eltype_tri<4>>(multi_carray<realnum, 15, 2> const& _nodepos);

    template<>
    realnum calc_region_integral<eltype_tri<5>>(multi_carray<realnum, 21, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 3>
    calc_region_integral_N<eltype_tri<1>>(multi_carray<realnum, 3, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 6>
    calc_region_integral_N<eltype_tri<2>>(multi_carray<realnum, 6, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 10>
    calc_region_integral_N<eltype_tri<3>>(multi_carray<realnum, 10, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 15>
    calc_region_integral_N<eltype_tri<4>>(multi_carray<realnum, 15, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 21>
    calc_region_integral_N<eltype_tri<5>>(multi_carray<realnum, 21, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 3, 2>
    calc_region_integral_dNdx<eltype_tri<1>>(multi_carray<realnum, 3, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 6, 2>
    calc_region_integral_dNdx<eltype_tri<2>>(multi_carray<realnum, 6, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 10, 2>
    calc_region_integral_dNdx<eltype_tri<3>>(multi_carray<realnum, 10, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 15, 2>
    calc_region_integral_dNdx<eltype_tri<4>>(multi_carray<realnum, 15, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 21, 2>
    calc_region_integral_dNdx<eltype_tri<5>>(multi_carray<realnum, 21, 2> const& _nodepos);

    template<>
    multi_carray<realnum, 3, 3, 2, 2>
    calc_region_integral_dNdx_dNdx<eltype_tri<1>>(multi_carray<realnum, 3, 2> const& _vertices);

    template<>
    multi_carray<realnum, 6, 6, 2, 2>
    calc_region_integral_dNdx_dNdx<eltype_tri<2>>(multi_carray<realnum, 6, 2> const& _vertices);

    template<>
    multi_carray<realnum, 10, 10, 2, 2>
    calc_region_integral_dNdx_dNdx<eltype_tri<3>>(multi_carray<realnum, 10, 2> const& _vertices);

    template<>
    multi_carray<realnum, 15, 15, 2, 2>
    calc_region_integral_dNdx_dNdx<eltype_tri<4>>(multi_carray<realnum, 15, 2> const& _vertices);

    template<>
    multi_carray<realnum, 21, 21, 2, 2>
    calc_region_integral_dNdx_dNdx<eltype_tri<5>>(multi_carray<realnum, 21, 2> const& _vertices);

    template<>
    multi_carray<realnum, 6, 3, 2>
    calc_region_integral_dNdx_L<eltype_tri<2>, eltype_tri<1>>(
        multi_carray<realnum, 6, 2> const& _nodepos1,
        multi_carray<realnum, 3, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 10, 3, 2>
    calc_region_integral_dNdx_L<eltype_tri<3>, eltype_tri<1>>(
        multi_carray<realnum, 10, 2> const& _nodepos1,
        multi_carray<realnum, 3, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 10, 6, 2>
    calc_region_integral_dNdx_L<eltype_tri<3>, eltype_tri<2>>(
        multi_carray<realnum, 10, 2> const& _nodepos1,
        multi_carray<realnum, 6, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 15, 3, 2>
    calc_region_integral_dNdx_L<eltype_tri<4>, eltype_tri<1>>(
        multi_carray<realnum, 15, 2> const& _nodepos1,
        multi_carray<realnum, 3, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 15, 6, 2>
    calc_region_integral_dNdx_L<eltype_tri<4>, eltype_tri<2>>(
        multi_carray<realnum, 15, 2> const& _nodepos1,
        multi_carray<realnum, 6, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 15, 10, 2>
    calc_region_integral_dNdx_L<eltype_tri<4>, eltype_tri<3>>(
        multi_carray<realnum, 15, 2> const& _nodepos1,
        multi_carray<realnum, 10, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 21, 3, 2>
    calc_region_integral_dNdx_L<eltype_tri<5>, eltype_tri<1>>(
        multi_carray<realnum, 21, 2> const& _nodepos1,
        multi_carray<realnum, 3, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 21, 6, 2>
    calc_region_integral_dNdx_L<eltype_tri<5>, eltype_tri<2>>(
        multi_carray<realnum, 21, 2> const& _nodepos1,
        multi_carray<realnum, 6, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 21, 10, 2>
    calc_region_integral_dNdx_L<eltype_tri<5>, eltype_tri<3>>(
        multi_carray<realnum, 21, 2> const& _nodepos1,
        multi_carray<realnum, 10, 2> const& _nodepos2);

    template<>
    multi_carray<realnum, 21, 15, 2>
    calc_region_integral_dNdx_L<eltype_tri<5>, eltype_tri<4>>(
        multi_carray<realnum, 21, 2> const& _nodepos1,
        multi_carray<realnum, 15, 2> const& _nodepos2);

}}}//---- namespace element fem sykes

#endif

