//-----------------------------------------------------------
//    fluid_utility.h
//-----------------------------------------------------------
#ifndef SYKES_ELEMENT_FLUID_UTILITY
#define SYKES_ELEMENT_FLUID_UTILITY

#include <cstddef>
#include "sykes/value_type.h"
#include "sykes/numeric/kronecker_delta.h"
#include "sykes/multi_carray.h"

namespace sykes{ namespace material{ namespace fluid{
    
    template<typename Eltype>
    multi_carray<realnum, Eltype::dim, Eltype::dim>
    calc_cauchy_stress(
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _nodepos,
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _velocity,
        realnum _p,    realnum _viscosity)
    {
        static int const dim = Eltype::dim;
        static int const elnodesize = Eltype::node_size;

        multi_carray<realnum, dim, dim, dim, dim>
            B = numeric::get_Bmat<dim>();
        multi_carray<realnum, elnodesize, dim>
            IdNdx = element::calc_region_integral_dNdx<Eltype>(_nodepos);
        realnum volume = fem::element::calc_region_integral<Eltype>(_nodepos);
        
        multi_carray<realnum, dim, dim> sigma;
        for(std::size_t i0 = 0; i0 < dim; ++i0){
        for(std::size_t i1 = 0; i1 < dim; ++i1){
            sigma[i0][i1] = 0.0;
            for(std::size_t i2 = 0; i2 < dim; ++i2){
            for(std::size_t i3 = 0; i3 < dim; ++i3){
            for(std::size_t i4 = 0; i4 < dim; ++i4){
            for(std::size_t i5 = 0; i5 < dim; ++i5){
            for(std::size_t n0 = 0; n0 < elnodesize; ++n0){
                sigma[i0][i1] += numeric::kdelta(i0, i1) * _p
                    + B[i2][i3][i4][i5] * dNdx[n0][i5] * _velocity[n0][i4];
            }}}}}
            sigma[i0][i1] /= volume;
        }}
        return sigma;
    }
    
}}}}//---- namespace fluid material sykes
#endif



