//-----------------------------------------------------------
//    solid_utility.h
//-----------------------------------------------------------
#ifndef SYKES_MATERIAL_SOLID
#define SYKES_LINEAR_ELASTIC_MATERIAL

#include <cstddef>
#include "sykes/multi_carray.h"
#include "sykes/value_type.h"
#include "sykes/numeric/kronecker_delta.h"

namespace sykes{ namespace material{
namespace solid{
    
    template<typename Eltype>
    multi_carray<realnum, Eltype::dim, Eltype::dim>
    calc_cauchy_strain(
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _nodepos,
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _nodedisp)
    {
        static std::size_t const dim = Eltype::dim;
        static std::size_t const elnodesize = Eltype::node_size;
        multi_carray<realnum, dim, dim> epsilon;
        auto B = numeric::get_Bmat<dim>();
        auto IdNdx = fem::element::calc_region_integral_dNdx<Eltype>(_nodepos);
        realnum volume = fem::element::calc_region_integral<Eltype>(_nodepos);
        
        for(std::size_t i0 = 0; i0 < dim; ++i0){
        for(std::size_t i1 = 0; i1 < dim; ++i1){
            epsilon[i0][i1] = 0.0;
            for(std::size_t i2 = 0; i2 < dim; ++i2){
            for(std::size_t i3 = 0; i3 < dim; ++i3){
            for(std::size_t i4 = 0; i4 < dim; ++i4){
            for(std::size_t i5 = 0; i5 < dim; ++i5){
            for(std::size_t n0 = 0; n0 < elnodesize; ++n0){
                epsilon[i0][i1] += B[i2][i3][i4][i5]
                    * IdNdx[n0][i5] * _nodedisp[n0][i4];
            }}}}}
            epsilon[i0][i1] /= volume;
        }}
        return epsilon;
    }
    
    template<typename Eltype>
    multi_carray<realnum, Eltype::dim, Eltype::dim>
    calc_cauchy_stress(
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _nodepos,
        multi_carray<realnum, Eltype::node_size, Eltype::dim> const& _nodedisp,
        multi_carray<realnum, Eltype::dim, Eltype::dim, Eltype::dim, Eltype::dim> const& _D)
    {
        static std::size_t const dim = Eltype::dim;
        static std::size_t const elnodesize = Eltype::node_size;
        auto B = numeric::get_Bmat<dim>();
        
        multi_carray<realnum, dim, dim> sigma;
        auto IdNdx = fem::element::calc_region_integral_dNdx<Eltype>(_nodepos);
        realnum volume = fem::element::calc_region_integral<Eltype>(_nodepos);
        
        
        for(std::size_t i0 = 0; i0 < dim; ++i0){
        for(std::size_t i1 = 0; i1 < dim; ++i1){
            sigma[i0][i1] = 0.0;
            for(std::size_t i2 = 0; i2 < dim; ++i2){
            for(std::size_t i3 = 0; i3 < dim; ++i3){
            for(std::size_t i4 = 0; i4 < dim; ++i4){
            for(std::size_t i5 = 0; i5 < dim; ++i5){
            for(std::size_t n0 = 0; n0 < elnodesize; ++n0){
                sigma[i0][i1] += _D[i0][i1][i2][i3] * B[i2][i3][i4][i5]
                    * IdNdx[n0][i5] * _nodedisp[n0][i4];
            }}}}}
            sigma[i0][i1] /= volume;
        }}
        return sigma;
    }
}}}//---- namespace solid material fem sykes
#endif



