//-----------------------------------------------------------
//    multi_carray.h
//-----------------------------------------------------------
#ifndef SYKES_MULTI_CARRAY
#define SYKES_MULTI_CARRAY

#include <cstddef>
#include <iterator>

namespace sykes{
    
    template<typename MArray>
    struct multi_carray_size;
    
    //-----------------------------------------------------------
    //    struct multi_carray
    //-----------------------------------------------------------
    template<typename Tp, std::size_t ... N>
    struct multi_carray;

    template<typename Tp, std::size_t N0>
    struct multi_carray<Tp, N0>
    {
        typedef Tp array_type[N0];
        typedef multi_carray self_type;
        typedef Tp value_type;
        typedef Tp& reference;
        typedef Tp const& const_reference;
        typedef std::size_t size_type;
        typedef Tp* pointer;
        typedef Tp const* const_pointer;
        array_type m_value;
        
        //---- element access
        Tp& operator[](std::size_t _pos)
        { return (m_value[_pos]); }
        Tp const& operator[](std::size_t _pos) const
        { return (m_value[_pos]); }
        
        //---- iterator support
        typedef Tp* iterator;
        typedef Tp const* const_iterator;
        typedef std::reverse_iterator<iterator> reverse_iterator;
        typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
        
        iterator begin()
        { return iterator((pointer)m_value); }
        const_iterator begin() const
        { return const_iterator((pointer)m_value); }
        iterator end()
        { return iterator(((pointer)m_value) + multi_carray_size<self_type>::value); }
        const_iterator end() const
        { return const_iterator(((pointer)m_value) + multi_carray_size<self_type>::value); }
        
        iterator rbegin()
        { return reverse_iterator(end()); }
        const_iterator rbegin() const
        { return const_iterator(end()); }
        reverse_iterator rend()
        { return reverse_iterator(rend()); }
        const_reverse_iterator rend() const
        { return const_reverse_iterator(begin()); }
    };

    template<typename Tp, std::size_t N0, std::size_t N1, std::size_t ... N2>
    struct multi_carray<Tp, N0, N1, N2...>
    {
        typedef multi_carray self_type;
        typedef typename multi_carray<Tp, N1, N2...>::array_type prior_array_type;
        typedef prior_array_type array_type[N0];
        typedef Tp value_type;
        typedef Tp& reference;
        typedef Tp const& const_reference;
        typedef std::size_t size_type;
        typedef Tp* pointer;
        typedef Tp const* const_pointer;
        array_type m_value;
    
        //---- element access
        prior_array_type& operator[](std::size_t _pos)
        { return (m_value[_pos]); }
        prior_array_type const& operator[](std::size_t _pos) const
        { return (m_value[_pos]); }
        
        //---- iterator support
        typedef Tp* iterator;
        typedef Tp const* const_iterator;
        typedef std::reverse_iterator<iterator> reverse_iterator;
        typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
        
        iterator begin()
        { return iterator((pointer)m_value); }
        const_iterator cbegin() const
        { return const_iterator((pointer)m_value); }
        iterator end()
        { return iterator(((pointer)m_value) + multi_carray_size<self_type>::value); }
        const_iterator cend() const
        { return const_iterator(((pointer)m_value) + multi_carray_size<self_type>::value); }
        
        iterator rbegin()
        { return reverse_iterator(end()); }
        const_iterator crbegin() const
        { return const_iterator(end()); }
        reverse_iterator rend()
        { return reverse_iterator(rend()); }
        const_reverse_iterator crend() const
        { return const_reverse_iterator(begin()); }
    };
    
    //-----------------------------------------------------------
    //    struct multi_carray_size
    //-----------------------------------------------------------
    template<typename Tp, std::size_t N0>
    struct multi_carray_size<multi_carray<Tp, N0> >
    {
        static std::size_t const value = N0;
    };
    
    template<typename Tp, std::size_t N0, std::size_t N1, std::size_t ... N2>
    struct multi_carray_size<multi_carray<Tp, N0, N1, N2...> >
    {
        static std::size_t const value =
            multi_carray_size<multi_carray<Tp, N1, N2...> >::value * N0;
    };
    
}//---- namespace sykes
#endif





