//-----------------------------------------------------------
//    kronecker_delta.h
//-----------------------------------------------------------
#ifndef SYKES_NUMERIC_KRONECKER_DELTA
#define SYKES_NUMERIC_KRONECKER_DELTA

#include <cstddef>
#include "sykes/multi_carray.h"
#include "sykes/value_type.h"

namespace sykes{ namespace numeric{
    
    template<typename Tp>
    realnum kdelta(Tp _n1, Tp _n2)
    {
        if(_n1 == _n2) return 1.0;
        else return 0.0;
    }
    
    template<std::size_t Dim>
    multi_carray<realnum, Dim, Dim, Dim, Dim>
    get_Bmat();
    
    template<>
    inline multi_carray<realnum, 2, 2, 2, 2>
    get_Bmat<2>()
    {
        return multi_carray<realnum, 2, 2, 2, 2>{{
            {{{1.0, 0.0}, {0.0, 0.0}}, {{0.0, 0.5}, {0.5, 0.0}}},
            {{{0.0, 0.5}, {0.5, 0.0}}, {{0.0, 0.0}, {0.0, 1.0}}}
        }};
    }
    
    template<>
    inline multi_carray<realnum, 3, 3, 3, 3>
    get_Bmat<3>()
    {
        return multi_carray<realnum, 3, 3, 3, 3>{{
            {{{1.0, 0.0, 0.0}, {0.0, 0.0, 0.0}, {0.0, 0.0, 0.0}},
             {{0.0, 0.5, 0.0}, {0.5, 0.0, 0.0}, {0.0, 0.0, 0.0}},
             {{0.0, 0.0, 0.5}, {0.0, 0.0, 0.0}, {0.5, 0.0, 0.0}}},
            {{{0.0, 0.5, 0.0}, {0.5, 0.0, 0.0}, {0.0, 0.0, 0.0}},
             {{0.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {0.0, 0.0, 0.0}},
             {{0.0, 0.0, 0.0}, {0.0, 0.0, 0.5}, {0.0, 0.5, 0.0}}},
            {{{0.0, 0.0, 0.5}, {0.0, 0.0, 0.0}, {0.5, 0.0, 0.0}},
             {{0.0, 0.0, 0.0}, {0.0, 0.0, 0.5}, {0.0, 0.5, 0.0}},
             {{0.0, 0.0, 0.0}, {0.0, 0.0, 0.0}, {0.0, 0.0, 1.0}}}
        }};
    }
    
}}//---- namespace numeric sykes
#endif

