//-----------------------------------------------------------
//    utility.h
//-----------------------------------------------------------
#ifndef SYKES_UTILITY
#define SYKES_UTILITY

#include <type_traits>
#include <tuple>
#include <cstddef>
#include "value_type.h"

namespace sykes{
    //-----------------------------------------------------------
    //    factorial
    //-----------------------------------------------------------
    template<std::size_t N>
    struct factorial;

    template<>
    struct factorial<0>
    {
        static std::size_t const value = 1;
    };
    
    template<std::size_t N>
    struct factorial
    {
        static std::size_t const value = factorial<N - 1>::value * N;
    };

    //-----------------------------------------------------------
    //    combination
    //-----------------------------------------------------------
    template<std::size_t N, std::size_t C>
    struct combination;

    template<std::size_t N, std::size_t C>
    struct combination
    {
        static_assert(N >= C, "struct sykes::combination, N >= C");
        static std::size_t const value = factorial<N>::value / (factorial<C>::value * factorial<N - C>::value);
    };
    
    template<std::size_t N>
    struct combination<N, N>
    {
        static std::size_t const value = 1;
    };
    
    //-----------------------------------------------------------
    //    element_type_dimension
    //-----------------------------------------------------------
    template<typename Eltype>
    struct element_type_dimension;
    
    template<typename Eltype>
    struct element_type_dimension
    {
        static std::size_t const value = Eltype::dim;
    };
    
    template<typename Eltype1, typename Eltype2>
    struct element_type_dimension<std::tuple<Eltype1, Eltype2>>
    {
        static std::size_t const value = Eltype1::dim;
    };
    
    //-----------------------------------------------------------
    //    is_tuple
    //-----------------------------------------------------------
    template<typename Tp>
    struct is_tuple
        : public std::false_type {};
    
    template<typename ... Tp>
    struct is_tuple<std::tuple<Tp...>>
        : public std::true_type {};
}//---- namespace sykes
#endif












